import pathlib as std_pathlib
import sys

import pytest

if sys.version_info < (3, 10):
    import pathlib2 as pathlib  # pragma: nocover
else:
    pathlib = std_pathlib  # pragma: nocover


@pytest.fixture
def tmp_path(tmp_path: std_pathlib.Path) -> pathlib.Path:
    """
    Override tmp_path to wrap in a more modern interface.
    """
    return pathlib.Path(tmp_path)



from __future__ import annotations

import pytest

from jaraco.text import join_continuation


def test_join_continuation_typing() -> None:
    """
    Ensures that the type annotation for join_continuation matches what we expect at runtime,
    since checkers seem to give a false-positive on supporting ``__iter__`` and ``__next__``.
    """

    good: list[str] = ['foo \\', 'bar', 'baz']
    expected: list[str] = ['foobar', 'baz']

    # Ensure consistency at runtime for different iterables and no type error
    assert list(join_continuation(good)) == expected
    assert list(join_continuation(tuple(good))) == expected
    assert list(join_continuation(map(lambda string: string, good))) == expected
    assert list(join_continuation(iter(good))) == expected
    assert list(join_continuation({string: None for string in good})) == expected

    # NOTE: str isn't an expected use-case
    # but unfortunately a str is an iterable of str, so we can't validate

    # iter supports SupportsGetItem[int, str], but we don't !
    bad: dict[int, str] = {i: string for (i, string) in enumerate(good)}
    with pytest.raises(AttributeError):
        list(join_continuation(bad))  # type: ignore[arg-type] # Testing for type error here!



from __future__ import annotations

import inflect
import typer
from more_itertools import always_iterable

import jaraco.text


def report_newlines(input: typer.FileText) -> None:
    r"""
    Report the newlines in the indicated file.

    >>> tmp_path = getfixture('tmp_path')
    >>> filename = tmp_path / 'out.txt'
    >>> _ = filename.write_text('foo\nbar\n', newline='', encoding='utf-8')
    >>> report_newlines(filename)
    newline is '\n'
    >>> filename = tmp_path / 'out.txt'
    >>> _ = filename.write_text('foo\nbar\r\n', newline='', encoding='utf-8')
    >>> report_newlines(filename)
    newlines are ('\n', '\r\n')
    """
    newlines = jaraco.text.read_newlines(input)
    count = len(tuple(always_iterable(newlines)))
    engine = inflect.engine()
    print(
        # Pyright typing issue: jaraco/inflect#210
        engine.plural_noun("newline", count),
        engine.plural_verb("is", count),
        repr(newlines),
    )


__name__ == '__main__' and typer.run(report_newlines)  # type: ignore[func-returns-value]



from __future__ import annotations

from typing import TYPE_CHECKING, Union

if TYPE_CHECKING:
    from _typeshed import SupportsGetItem, SupportsRead
    from typing_extensions import TypeAlias

    # Same as builtins._TranslateTable from typeshed
    _TranslateTable: TypeAlias = SupportsGetItem[int, Union[str, int, None]]

qwerty = "-=qwertyuiop[]asdfghjkl;'zxcvbnm,./_+QWERTYUIOP{}ASDFGHJKL:\"ZXCVBNM<>?"
dvorak = "[]',.pyfgcrl/=aoeuidhtns-;qjkxbmwvz{}\"<>PYFGCRL?+AOEUIDHTNS_:QJKXBMWVZ"


to_dvorak = str.maketrans(qwerty, dvorak)
to_qwerty = str.maketrans(dvorak, qwerty)


def translate(input: str, translation: _TranslateTable) -> str:
    """
    >>> translate('dvorak', to_dvorak)
    'ekrpat'
    >>> translate('qwerty', to_qwerty)
    'x,dokt'
    """
    return input.translate(translation)


def _translate_stream(stream: SupportsRead[str], translation: _TranslateTable) -> None:
    """
    >>> import io
    >>> _translate_stream(io.StringIO('foo'), to_dvorak)
    urr
    """
    print(translate(stream.read(), translation))
